@@ -8,7 +8,8 @@
 #' The functions \code{pkg_attach2()} and \code{pkg_load2()} are wrappers of
 #' \code{pkg_attach(install = TRUE)} and \code{pkg_load(install = TRUE)},
 #' respectively. \code{loadable()} is an abbreviation of
-#' \code{requireNamespace(quietly = TRUE)}.
+#' \code{requireNamespace(quietly = TRUE)}. \code{pkg_available()} tests if a
+#' package with a minimal version is available.
 #'
 #' These are convenience functions that aim to solve these common problems: (1)
 #' We often need to attach or load multiple packages, and it is tedious to type
@@ -16,13 +17,22 @@
 #' packages when attaching/loading them but they have not been installed.
 #' @param ... Package names (character vectors, and must always be quoted).
 #' @param install Whether to automatically install packages that are not
-#'   available using \code{\link{install.packages}()}. You are recommended to
-#'   set a CRAN mirror in the global option \code{repos} via
-#'   \code{\link{options}()} if you want to automatically install packages.
+#'   available using \code{\link{install.packages}()}. Besides \code{TRUE} and
+#'   \code{FALSE}, the value of this argument can also be a function to install
+#'   packages (\code{install = TRUE} is equivalent to \code{install =
+#'   install.packages}), or a character string \code{"pak"} (equivalent to
+#'   \code{install = pak::pkg_install}, which requires the \pkg{pak} package).
+#'   You are recommended to set a CRAN mirror in the global option \code{repos}
+#'   via \code{\link{options}()} if you want to automatically install packages.
 #' @param message Whether to show the package startup messages (if any startup
 #'   messages are provided in a package).
 #' @return \code{pkg_attach()} returns \code{NULL} invisibly. \code{pkg_load()}
 #'   returns a logical vector, indicating whether the packages can be loaded.
+#' @seealso \code{pkg_attach2()} is similar to \code{pacman::p_load()}, but does
+#'   not allow non-standard evaluation (NSE) of the \code{...} argument, i.e.,
+#'   you must pass a real character vector of package names to it, and all names
+#'   must be quoted. Allowing NSE adds too much complexity with too little gain
+#'   (the only gain is that it saves your effort in typing two quotes).
 #' @import utils
 #' @export
 #' @examples library(xfun)
@@ -37,7 +47,7 @@ pkg_attach = function(
     suppressPackageStartupMessages(base::library(...))
   }
   for (i in c(...)) {
-    if (install && !loadable(i)) pkg_install(i)
+    if (!isFALSE(install) && !loadable(i)) pkg_install(i, install)
     library(i, character.only = TRUE)
   }
 }
@@ -50,8 +60,8 @@ pkg_load = function(..., error = TRUE, install = FALSE) {
   if (n == 0) return(invisible(res))
   for (i in seq_len(n)) {
     res[i] = loadable(p <- pkg[i])
-    if (install && !res[i]) {
-      pkg_install(p); res[i] = loadable(p)
+    if (!isFALSE(install) && !res[i]) {
+      pkg_install(p, install); res[i] = loadable(p)
     }
   }
   if (error && any(!res)) stop('Package(s) not loadable: ', paste(pkg[!res], collapse = ' '))
@@ -79,6 +89,14 @@ loadable = function(pkg, strict = TRUE, new_session = FALSE) {
   }
 }
 
+#' @param version A minimal version number. If \code{NULL}, only test if a
+#'   package is available and do not check its version.
+#' @rdname pkg_attach
+#' @export
+pkg_available = function(pkg, version = NULL) {
+  loadable(pkg) && (is.null(version) || packageVersion(pkg) >= version)
+}
+
 #' @rdname pkg_attach
 #' @export
 pkg_attach2 = function(...) pkg_attach(..., install = TRUE)
@@ -87,6 +105,9 @@ pkg_attach2 = function(...) pkg_attach(..., install = TRUE)
 #' @export
 pkg_load2 = function(...) pkg_load(..., install = TRUE)
 
+pkg_update = function(...) {
+  update.packages(ask = FALSE, checkBuilt = TRUE, ...)
+}
 
 broken_packages = function(reinstall = TRUE) {
   pkgs = unlist(plapply(.packages(TRUE), function(p) if (!loadable(p)) p))
@@ -123,7 +144,7 @@ install_dir = function(src, build = TRUE, build_opts = NULL, install_opts = NULL
 }
 
 install_brew_deps = function(pkg = .packages(TRUE)) {
-  con = url('https://macos.rbind.org/bin/macosx/sysreqsdb.rds')
+  con = url('https://macos.rbind.io/bin/macosx/sysreqsdb.rds')
   on.exit(close(con), add = TRUE)
   inst = installed.packages()
   pkg = intersect(pkg, pkg_needs_compilation(inst))
@@ -168,3 +189,12 @@ reinstall_from_cran = function(dry_run = TRUE, skip_github = TRUE) {
 }
 
 base_pkgs = function() rownames(installed.packages(priority = 'base'))
+
+# update one package (from source by default)
+pkg_update_one = function(pkg, type = 'source') {
+  opts = options(repos = c(CRAN = 'https://cran.r-project.org'))
+  on.exit(options(opts), add = TRUE)
+  if (is.null(pkgs <- old.packages(type = type)) || !pkg %in% rownames(pkgs)) return()
+  install.packages(pkg, pkgs[pkg, 'LibPath'], type = type, INSTALL_opts = '--no-staged-install')
+  NULL
+}
